#include <math.h>
#include <matrix.h>
#include <mex.h>
#include <iostream>     // std::cout
#include <algorithm>    // std::max
#include <stdio.h>      // sin
#define PI 3.1415926535897932
#include <cmath>   
void evalgradf(double *T,double *X, int *D, int m, int x_i, double *FX);
void evalfullgradf(double *T,double *X, int *D, int m, double *FX);

void mexFunction(int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[]) {
    
    double *X;
    double *T;
    double *Dt;
    // int x_i;
    int i;
    int m;           
    double *Out;      // output matrix //
    
    T = mxGetPr(prhs[0]);
    X = mxGetPr(prhs[1]);
    Dt = mxGetPr(prhs[2]); 
    m = mxGetN(prhs[2]); // get the size of D = order of the tensor
    // x_i = static_cast<int>(mxGetScalar(prhs[3]));
    
    int *D = new int[m];
    // mexPrintf("this the %i",m);
    int N;
    N=0;
    for(i=0;i<m;++i){
        D[i] = static_cast<int>(Dt[i]);
        N = N+D[i];
    }
    plhs[0] = mxCreateDoubleMatrix(1,N,mxREAL);
    Out = mxGetPr(plhs[0]);
    for(i = 0; i < N; ++i){
            Out[i] = 0;
    }
    evalfullgradf(T,X,D,m,Out);

    
    delete[] D;
}

void evalfullgradf(double *T,double *X, int *D, int m, double *FX){
    int i;
    int mu;
    int j;
    int jj;
    int *I = new int[m];
    int *Sx = new int[m];
    int N;
    double *Prodx = new double[m];
    N=1;
    Sx[0]=0;
    for(i=0;i<m;++i){
        N = N*D[i];
        I[i]=0;
        if(i<m-1){
            Sx[i+1]=Sx[i]+D[i];
        }
    }
    for(i=0;i<N;++i){
        // prepare the product of X_{1,j_1}*...*X_{i-1,j_{i-1}}*X_{i+1,j_{i+1}}*...*X_{m,j_m} for jj = 1,..,m
       for(j=0;j<m;++j){
            Prodx[j] = 1;
        }
       
        for(j=0; j<m;++j){
            for(jj=0;jj<m;++jj){
                if(jj!=j){
                    Prodx[jj] = Prodx[jj]*X[Sx[j]+I[j]];
                }
            }
        }
       
        // update the output
        for(jj=0;jj<m;++jj){
            FX[Sx[jj]+I[jj]]=FX[Sx[jj]+I[jj]]+T[i]*Prodx[jj];
        }
       
        // update the indexes j_1,...,j_m
        I[0]=I[0]+1;
        if(I[0]==D[0]){
            I[0]=0;
            I[1]=I[1]+1;
            mu = 1;
            while(I[mu]==D[mu] && mu<m){
                I[mu]=0;
                if(mu+1<m){
                    I[mu+1]=I[mu+1]+1;
                }
                mu=mu+1;
            }
        }
    }
    delete[] I;
    delete[] Sx;
    delete[] Prodx;
}






void evalgradf(double *T,double *X, int *D, int m, int x_i, double *FX){
    int i;
    int mu;
    int j;
    int *I = new int[m];
    int *Sx = new int[m];
    int N;
    N=1;
    Sx[0]=0;
    double prodx;
    for(i=0;i<m;++i){
        N = N*D[i];
        I[i]=0;
        if(i<m-1){
            Sx[i+1]=Sx[i]+D[i];
        }
    }
    for(i=0;i<N;++i){
        prodx = 1;
        // prepare the product of X_{1,j_1}*...*X_{i-1,j_{i-1}}*X_{i+1,j_{i+1}}*...*X_{m,j_m}
        for(j=0; j<m;++j){
            if(j!= x_i){
                prodx = prodx*X[Sx[j]+I[j]];
             }
        }
        // update the output
        FX[I[x_i]]=FX[I[x_i]]+T[i]*prodx;
        // update the indexes j_1,...,j_m
        I[0]=I[0]+1;
        if(I[0]==D[0]){
            I[0]=0;
            I[1]=I[1]+1;
            mu = 1;
            while(I[mu]==D[mu] && mu<m){
                I[mu]=0;
                if(mu+1<m){
                    I[mu+1]=I[mu+1]+1;
                }
                mu=mu+1;
            }
        }
    }
    delete[] I;
    delete[] Sx;
}